# data preprocessing of wt/PiZ mutant mice liver expression (RNA-seq) data
# input: raw transcript-level counts in .txt files per sample
# output: n x (m+1) matrix of discretized gene expression values for zinc transporter genes
gse <- "GSE141593"
DE <- FALSE #whether or not to include top differentially expressed genes in addition to zn tp genes
n.de.genes <- 25
# libraries and functions ---------------------------------------------------------------
library(DESeq2)
library(openxlsx)

discretize_gene_vector <- function(gene_vector){
  
  is_low <- gene_vector < -1
  is_high <- gene_vector > 1
  is_none <- !is_low & !is_high
  
  gene_vector[is_none] <- "noChange"
  gene_vector[is_low] <- "low"
  gene_vector[is_high] <- "high"
  
  return(gene_vector)
}

# read in files and merge samples into one matrix -------------------------
count.files <- list.files("./data/", pattern = "counts.txt")
gsm <- gsub(pattern = "_.*", replacement = "", count.files)

# read sample files into a list
individual.files <- list()
for (f in 1:length(count.files)){
  filepath <- paste0("./data/", count.files[f])
  tmp <- read.table(filepath)
  
  # give the count column a unique name for merging later
  colnames(tmp) <- c("transcript.id", gsm[f])
  individual.files[[f]] <- tmp
  
}

# merge samples by transcript id column
merged <- Reduce(function(x,y) merge(x,y, all = TRUE, by = "transcript.id"), individual.files)

# remove first 5 rows since we don't need them
merged <- merged[-c(1:5),]

# move transcript.id column to rownames and remove
rownames(merged) <- merged$transcript.id
merged <- merged[,-1]
head(merged)

# normalize and standardize count data ------------------------------------

# make a quick metadata table for DESeq2
metadata <- data.frame(sampleID = gsm,
                       treatment = c(rep("wild type", 5), rep("PiZ mutant", 5)))
metadata$treatment <- factor(metadata$treatment, levels = c("wild type", "PiZ mutant"), labels = c("wt", "piz"))

# create DESeq object from count matrix, filter low expression genes and estimate size factors
diffxp <- DESeqDataSetFromMatrix(countData = merged,
                                 colData = metadata,
                                 design = ~ treatment)

keep <- rowSums(counts(diffxp) > 10) >= 3
diffxp <- diffxp[keep,]
diffxp <- estimateSizeFactors(diffxp)

#normalize and standardize
norm.counts <- counts(diffxp, normalized = TRUE)

standardized.counts <- scale(t(norm.counts))

# make sure its in the expected orientation
standardized.counts[1:5,1:5]


# discretize data ---------------------------------------------------------
discrete.counts <- apply(standardized.counts, MARGIN = 2, discretize_gene_vector)

# double check
discrete.counts[1:5,1:5]
standardized.counts[1:5,1:5]


# filter for genes of interest, and add treatment column ------------------
genes <- read.table("./data/zn_transporter_genes.txt", sep = "\n")$V1
dc.goi <- which(colnames(discrete.counts) %in% genes)

if(DE == TRUE){
  # perform a quick DE analysis
  diffxp <- DESeq(diffxp)
  diffxp.res <- results(diffxp)
  diffxp.LFC <- lfcShrink(diffxp, res = diffxp.res, coef = "treatment_piz_vs_wt")
  diffxp.LFC <- diffxp.LFC[order(abs(diffxp.LFC$pvalue)),]
  
  # take the top DE genes plus the genes of interest
  de.genes <- which(colnames(discrete.counts) %in% rownames(diffxp.LFC)[1:n.de.genes])
  
  goi <- union(dc.goi, de.genes)
  pizmice.counts <- data.frame(pizMutation = metadata$treatment,
                               as.data.frame(discrete.counts[,goi]))
}else{
  
  pizmice.counts <- data.frame(pizMutation = metadata$treatment,
                               as.data.frame(discrete.counts[,dc.goi]))
}



save(pizmice.counts, file = paste0("./analysis/bnlearn/", gse, "_zn_transporter_gene_expr.RData"))
write.xlsx(pizmice.counts, file = paste0("./data/", gse, "_zn_transporter_gene_expr_clean.xlsx"))

